import os
import sys
import array
from PIL import Image

SCREEN_WIDTH, SCREEN_HEIGHT = 184, 96  # Adjust based on your screen size
SIZE = (SCREEN_WIDTH, SCREEN_HEIGHT)
FRAME_DURATION = 1000 // 21.5  # 30 FPS -> 33ms per frame

def unpack16bitRGB(word):
    """Extracts RGB values from a 16-bit packed word and scales them to 8-bit."""
    r = (word >> 11) & 0x1F
    g = (word >> 5) & 0x3F
    b = word & 0x1F
    return (int(r * 255 / 31), int(g * 255 / 63), int(b * 255 / 31))

def convert_raw_to_frame(raw_data):
    """Converts raw pixel data into a PIL image."""
    img = Image.new('RGB', (SCREEN_WIDTH, SCREEN_HEIGHT))
    pixels = img.load()
    
    for y in range(SCREEN_HEIGHT):
        for x in range(SCREEN_WIDTH):
            index = y * SCREEN_WIDTH + x
            pixels[x, y] = unpack16bitRGB(raw_data[index])
    
    return img

def convert_raw_to_gif(raw_file, out_gif):
    """Reads a .raw file and converts it into an animated GIF at 30fps."""
    with open(raw_file, "rb") as f:
        raw_data = array.array("H")
        raw_data.fromfile(f, os.path.getsize(raw_file) // 2)
    
    num_frames = len(raw_data) // (SCREEN_WIDTH * SCREEN_HEIGHT)
    if len(raw_data) % (SCREEN_WIDTH * SCREEN_HEIGHT) != 0:
        print("Warning: The raw file size is not a perfect multiple of the expected frame size.")
    
    frames = [convert_raw_to_frame(raw_data[i * SCREEN_WIDTH * SCREEN_HEIGHT:(i + 1) * SCREEN_WIDTH * SCREEN_HEIGHT]) for i in range(num_frames)]
    
    if frames:
        frames[0].save(out_gif, save_all=True, append_images=frames[1:], loop=0, duration=FRAME_DURATION)
        print(f'GIF saved as {out_gif}')
    else:
        print("Error: No frames to save.")

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python script.py <input.raw>")
        sys.exit(1)
    
    input_path = sys.argv[1]
    if os.path.isfile(input_path) and input_path.endswith('.raw'):
        convert_raw_to_gif(input_path, 'output.gif')
    else:
        print("Error: The provided argument is not a valid .raw file.")
